function [sirf, dirf, iirf, sirfacc, dirfacc, iirfacc,...
    fevd1, fevd2, fevd3, ind, d] = GIRF_cholesky(sd, aa, qd, s1, s2, vd, om,...
    YS, HOR, N, L, SC)
% This function estimates generalised impulse response functions following
% Koop et al (1996) and uses a cholesky decomposition. Algorithm used
% loosely follows Rubio-Ramirez et al (2010).
% Code is based on Baumeister and Peersma (2013) JAE and Baumeister and 
% Peersman (2013) AEJ:Macroeconomics. In particular after computing the
% GIRFs, we compute the FEVDs as in Baumeister and Peersman (2013) AEJ.

% Michael Ellington 14/01/2017
% Step 1: Simulate VAR elements into the future
%        1) Simulate VAR covariance matrix
% Shocks to A_{t} matrix
% NOTE: since we are simulating the responses in each point in time A_{t} =
% A at each observation.
aa_e = real(sqrtm([s1 zeros(1,2); zeros(2,1) s2]))*randn((N*(N-1))/2,HOR+L+1);
% This puts shocks in the block diagonal matrix S (see Eq(9) in paper)
% Now simulate elements of matrix through the horizon
for jj = 1:HOR+L+1
    aa(:,:,jj+1) = aa(:,:,jj) + aa_e(:,jj);
end

% Shocks to the H_{t} matrix 
om_e = diag(vd)*randn(N,HOR+L+1);
% Simulate H_{t} matrix over the horizon
om = log(om);
for jj = 1:HOR+L+1
    om(:,:,jj+1) = om(:,:,jj) + om_e(:,jj);
end
% Compute the VARs reduced-form covariance matrix
VAR=zeros(N,N,HOR+L+1);
for jj = 1:HOR+L+1
    iA = inv(chofac(N, aa(:,:,jj)));
    VAR(:,:,jj) = iA*diag(om(:,:,jj))*iA'; % \Omega_{t}
end
VARC = VAR(:,:,1);
% 2) Simulate the theta matrix (i.e. the \Beta_{t})
SI = sd;
B = zeros(N,N*L+1,HOR+L+1);
jj = 2;
trial=0;
maxtrial = 50;
while jj <= HOR+L+1
    
    dd = sd(:,:,jj-1) + real(mysqrt_varm(qd))*randn(N*(1+N*L),1); 
    b = [dd(1:1+N*L,1)'; dd((1+N*L)+1:2*(1+N*L),1)';...
        dd(2*(1+N*L)+1:3*(1+N*L),1)'];
        
        if SC == 0
        sd(:,:,jj) = dd;
        B(:,:,jj) = b;
        jj = jj+1;
        else
            if max(abs(varroots(L,N,b))) < 1
                sd(:,:,jj) = dd;
                B(:,:,jj) = b;
                jj = jj+1;
                trial = 0;
            elseif max(abs(varroots(L,N,b))) > 1 && trial < maxtrial
                trial = trial + 1;
            elseif max(abs(varroots(L,N,b))) > 1 && trial== maxtrial
               sirf=[]; dirf=[]; iirf=[]; sirfacc=[]; dirfacc=[]; iirfacc=[]; ...
               fevd1=[]; fevd2=[]; fevd3=[];  ind=0; d=0;
               return
            end
        end
end
% Lines 46:72 simulate the shocks hitting the beta matrix we then put the
% shocked coefficients into the matrix B which has equations in the rows
% and parameters in the columns. the pages represent the "forecast horizon"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2: GIRFs                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

a0 = chol(VARC');
Y_B = [YS zeros(N, HOR+(L-1)-L)]; % if YS(:,tt-L:tt-1) and tt=5 to start YS(:,1:4) if N= 3, L=4
    % then Y_B is 3x24 matrix with zeros in each row from column 5
    % "HOR+(L-1)-L" places a matrix of zeros with N rows through the
    % forecast horizon.
    Y_SUP = Y_B;
    Y_DEM = Y_B;
    Y_MOP = Y_B;

    SHOCKS = randn(N, HOR);
    % Calculate the impact of the shocks:
    Y_B(:,L+1) = a0*SHOCKS(:,1); % Y_B is Y_baseline where no shock occurs
    Y_SUP(:,L+1) = a0*[1+SHOCKS(1,1) SHOCKS(2:3,1)']'; % epsilon_1,i +1 is the first shock
    % a0 is 3x3 SHOCKS vector is 3x1 therefore the value in Y_SUP(:,L+1)
    % is the dot product of a0 and the shock vector where the first element
    % in SHOCKS vector is 1 + the random draw
    Y_DEM(:,L+1) = a0*[SHOCKS(1,1) 1+SHOCKS(2,1) SHOCKS(3,1)]';
    Y_MOP(:,L+1) = a0*[SHOCKS(1:2,1)' 1+SHOCKS(3,1) ]';
    for tt = 2+L:HOR+L
        shocks = mysqrt_varm(VAR(:,:,tt-L))*SHOCKS(:,tt-L);
        % Take the eigenvalue eigenvector deocmposition of the variance
        % covariance matrix. VCV matrix is symmetric and positive definite
        % therefore  asquare root always exists. Taking the square root of
        % the eigenvalues always exists. eigenvalue eigenvector decomposit
        % -ion implies [Q,T] = eig(A), Q contains eigenvectors, and T contains the eigenvalues 
        % A=Q*T*Q' R=diag(sqrt(diag(T)); takes the square root of the eigenvalues
        % thus A^{0.5} = Q*R*Q';
        Y_B(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_B(:,tt-L:tt-1)))] + shocks;
        % if N=3, L=4 B(:,:,tt-L) is 3x13. let vec1 = 
        % [1; myvec(myfliplr(Y_B(:,tt-L:tt-1)))] denote the vector of lagged
        % dependent values at this point in time as well as the structural
        % impact matrix. myfliplr is essentially the same as fliplr and
        % flips the columns from left to right meaning that the structural
        % impact matrix are in rows 2:4 (essentially Y(t-1), row 1 is the 
        % constant. rows 5:7 Y(t-2), are observed Y values, 8:10 Y(t-3),
        % 11:13 Y(t-4). myvec does the same as A=A(:) stacking columns into
        % a vector. The dot product of B*vec1 is 3x1 adding shocks which is
        % also 3x1 results in Y_B(:,tt) being 3x1. This is the conditional
        % forecast at horizon tt under the scenario that no shock hits the
        % system.
        Y_SUP(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_SUP(:,tt-L:tt-1)))] + shocks;
        Y_DEM(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_DEM(:,tt-L:tt-1)))] + shocks;
        Y_MOP(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_MOP(:,tt-L:tt-1)))] + shocks;

        % The above are the conditional forecasts of variables in the VAR
        % when a shock hits the system.
    end
        Y_B = Y_B(:,L+1:end);
        Y_SUP = Y_SUP(:,L+1:end);
        Y_DEM = Y_DEM(:,L+1:end);
        Y_MOP = Y_MOP(:,L+1:end);
        
        % IRF is the differences between the conditional forecast of VAR
        % when a shock hits and the conditional forecast of when the shock
        % does not hit.
        
        % Impulses affected by ordering of variables. Do not need to check
        % sign restrictions. Therefore, monetary policy shock is third in
        % the recursive ordering. 
        
        irf1 = Y_SUP - Y_B; % supply
        irf2 = Y_DEM - Y_B; % demand
        irf3 = Y_MOP - Y_B; % monetary policy
        
        % accumulated shocks
        ir1 = cumsum(irf1,2);
        ir2 = cumsum(irf2,2);
        ir3 = cumsum(irf3,2);
        
        % Supply
        irs1 = irf1(1,1:HOR);
        irs2 = irf1(2,1:HOR);
        irs3 = irf1(3,1:HOR);
        sirf = [irs1' irs2' irs3'];
        
        % Demand
        ird1 = irf2(1,1:HOR);
        ird2 = irf2(2,1:HOR);
        ird3 = irf2(3,1:HOR);
        dirf = [ird1' ird2' ird3'];
        
        % Monetary Policy
        iri1 = irf3(1,1:HOR);
        iri2 = irf3(2,1:HOR);
        iri3 = irf3(3,1:HOR);

        iirf = [iri1' iri2' iri3'];
        
        % Accumulated
        irfs1 = ir1(1,1:HOR);
        irfs2 = ir1(2,1:HOR);
        irfs3 = ir1(3,1:HOR);
        sirfacc = [irfs1' irfs2' irfs3'];
        
        irfd1 = ir2(1,1:HOR);
        irfd2 = ir2(2,1:HOR);
        irfd3 = ir2(3,1:HOR);
        dirfacc = [irfd1' irfd2' irfd3'];
        
        irfi1 = ir3(1,1:HOR);
        irfi2 = ir3(2,1:HOR);
        irfi3 = ir3(3,1:HOR);
        iirfacc = [irfi1' irfi2' irfi3'];
        
        
        % FORECAST ERROR VARIANCES
        decomp = vardecomp(N,HOR,irf);
        des1 = decomp(1,1,1:HOR);
        des2 = decomp(2,1,1:HOR);
        des3 = decomp(3,1,1:HOR);
        fevd1 = [des1' des2' des3'];
        ded1 = decomp(1,2,1:HOR);
        ded2 = decomp(2,2,1:HOR);
        ded3 = decomp(3,2,1:HOR);
        fevd2 = [ded1' ded2' ded3'];
        dei1 = decomp(1,3,1:HOR);
        dei2 = decomp(2,3,1:HOR);
        dei3 = decomp(3,2,1:HOR);
        fevd3 = [dei1' dei2' dei3'];        
        ind = 1; d = 1;
        
